package com.flutter.invitereferrals;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.util.Log;

import androidx.annotation.NonNull;

import com.invitereferrals.invitereferrals.IRInterfaces.IRTrackReferrerCode;
import com.invitereferrals.invitereferrals.IRInterfaces.IRTrackingCallback;
import com.invitereferrals.invitereferrals.IRInterfaces.InviteReferralsSharingInterface;
import com.invitereferrals.invitereferrals.IRInterfaces.OnUIBuild;
import com.invitereferrals.invitereferrals.InviteReferralsApi;
import com.invitereferrals.invitereferrals.InviteReferralsApplication;
import com.invitereferrals.invitereferrals.internal.IRCampaignData;
import com.invitereferrals.invitereferrals.internal.IREventData;
import com.invitereferrals.invitereferrals.internal.IRUserData;

import org.json.JSONObject;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * InviteReferralsPlugin
 */
public class InviteReferralsPlugin implements FlutterPlugin, MethodChannel.MethodCallHandler, ActivityAware {
    private MethodChannel channel;

    private static final String TAG = "InviteReferrals-Flutter";
    private static final String PLUGIN_VERSION = "1.2.0";
    private static final String INLINE_BUTTON = "inline_btn";
    private static final String LOGIN_SCREEN = "login_screen";
    private static final String TRACKING = "tracking";
    private static final String INVITE = "campaignPopup";
    private static final String WELCOME_MSG = "welcomeMessage";
    private static final String REFERRER_CODE = "getReferrerCode";
    private static final String SET_LANGUAGE = "setLocale";
    private static final String GET_SHARING_DETAILS = "getSharingDetails";
    private static final String HANDEL_DONE_BUTTON = "campaignCallbacksListener";
    private static final String GET_REFERRING_PARAMS = "getReferringParams";
    private static final String GET_LINK_INFO = "getLinkInfo";
    private static final String TRACK_INVITE = "trackInvite";
    private static final String SHOW_LOG = "showLog";
    private static final String CLEAR_USER_DETAILS = "clearUserDetails";

    private Context flutterContext;
    private Activity activity;

    boolean logSignal = false;

    public static void register(Context context) {
        Log.i(TAG, "REGISTER !!");
        Log.i(TAG, "PLUGIN_VERSION : " + PLUGIN_VERSION);
        try {
            InviteReferralsApplication.register((Application) context.getApplicationContext());
        } catch (Exception e) {
            Log.i(TAG, "REGISTER ERROR : " + e.toString());
        }
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        Log.i(TAG, "ON ATTACHED TO ENGINE !!");
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "flutter_invitereferrals");
        channel.setMethodCallHandler(this);
        flutterContext = flutterPluginBinding.getApplicationContext();
        fetchLogSignal();

    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        Log.i(TAG, "ON DETACHED FROM ENGINE !!");
        channel.setMethodCallHandler(null);
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding activityBinding) {
        Log.i(TAG, "ON ATTACHED TO ACTIVITY !!");
        activity = activityBinding.getActivity();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
        Log.i(TAG, "ON DETACHED FROM ACTIVITY FOR CONFIG CHANGES !!");
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding activityPluginBinding) {
        Log.i(TAG, "ON REATTACHED TO ACTIVITY FOR CONFIG CHANGES !!");
    }

    @Override
    public void onDetachedFromActivity() {
        Log.i(TAG, "ON DETACHED FROM ACTIVITY !!");
    }


    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        Log.i(TAG, "ON METHOD CALL !!");
        String action = call.method;
        switch (action) {
            case INLINE_BUTTON:
                sharing_screen(call);
                break;
            case LOGIN_SCREEN:
                login_screen(call);
                break;
            case TRACKING:
                tracking(call, result);
                break;
            case WELCOME_MSG:
                welcome_msg();
                break;
            case REFERRER_CODE:
                get_referrer_code(result);
                break;
            case SET_LANGUAGE:
                set_language(call);
                break;
            case INVITE:
                campaignPopup(call);
                break;
            case GET_SHARING_DETAILS:
                get_sharing_details(call, result);
                break;
            case HANDEL_DONE_BUTTON:
                campaignCallbacks();
                break;
            case GET_REFERRING_PARAMS:
                getReferringParams(result);
                break;
            case TRACK_INVITE:
                trackInvite(call);
                break;
            case GET_LINK_INFO:
                getLinkInfo();
                break;
            case CLEAR_USER_DETAILS:
                clearUserDetails();
                break;
            case SHOW_LOG:
                show_log(call);
                break;
            default:
                Log.e(TAG, "Invalid action : " + action);
                result.notImplemented();
        }
    }

    private void sharing_screen(MethodCall fetchData) {
        Log.i(TAG, "SHARING SCREEN !!");
        try {
            IRUserData.Builder userInfo = fetchUserInfo(fetchData);
            IRCampaignData.Builder campaignInfo = fetchCampaignInfo(fetchData);
            launch_campaign(campaignInfo, userInfo);
        } catch (Exception e) {
            Log.e(TAG, "SHARING SCREEN ERROR :" + e);
        }
    }

    private void login_screen(MethodCall fetchData) {
        Log.i(TAG, "LOGIN SCREEN !!");
        try {
            IRCampaignData.Builder campaignInfo = fetchCampaignInfo(fetchData);
            launch_campaign(campaignInfo, null);
        } catch (Exception e) {
            Log.e(TAG, "LOGIN SCREEN ERROR :" + e);
        }
    }

    private void tracking(MethodCall fetchData, final Result result) {
        Log.i(TAG, "TRACKING !!");
        String eventName = "";

        try {
            eventName = fetchData.argument("eventName");
            Log.i(TAG, "eventName : " + eventName);
        } catch (Exception e) {
            Log.e(TAG, "EVENT-NAME ERROR :" + e);
        }

        IREventData.Builder eventInfo = fetchEventInfo(fetchData);
        IRUserData.Builder userInfo = fetchUserInfo(fetchData);

        InviteReferralsApi.getInstance(flutterContext).tracking(eventInfo, userInfo);
        InviteReferralsApi.getInstance(flutterContext).onEventTrack(new IRTrackingCallback() {
            @Override
            public void onEventHit(JSONObject response) {
                result.success(response.toString());
            }
        });
    }

    private void campaignPopup(MethodCall fetchData) {
        Log.i(TAG, "CAMPAIGN POPUP !!");

        String ruleName = "";
        try {
            ruleName = fetchData.argument("rule");
            slg("Rule Name", ruleName);
        } catch (Exception e) {
            Log.e(TAG, "ERROR :" + e);
        }

        try {
            IRUserData.Builder userInfo = fetchUserInfo(fetchData);
            IRCampaignData.Builder campaignInfo = fetchCampaignInfo(fetchData);

            if (activity != null)
                InviteReferralsApi.getInstance(activity).campaignPopup(ruleName, userInfo, campaignInfo);
            else Log.i(TAG, "ACTIVITY PARAMETER CAN NOT BE NULL !!");

        } catch (Exception e) {
            Log.e(TAG, "CAMPAIGN POPUP ERROR :" + e);
        }
    }

    private void get_sharing_details(MethodCall fetchData, final Result result) {
        Log.i(TAG, "GET SHARING DETAILS !!");

        IRUserData.Builder userInfo = fetchUserInfo(fetchData);
        IRCampaignData.Builder campaignInfo = fetchCampaignInfo(fetchData);
        try {
            InviteReferralsApi.getInstance(flutterContext).getSharingDetails(new InviteReferralsSharingInterface() {
                @Override
                public void getShareData(JSONObject response) {
                    if (response != null) {
                        slg("GET SHARING DETAILS RESPONSE ", response.toString());
                        result.success(response.toString());
                    } else {
                        slg("GET SHARING DETAILS RESPONSE", "null");
                        result.success("null");
                    }

                }
            }, userInfo, campaignInfo);

        } catch (Exception e) {
            Log.e(TAG, "GET SHARING DETAIL LISTENER ERROR :" + e);
        }
    }

    private void welcome_msg() {
        Log.i(TAG, "WELCOME-MSG !!");
        try {
            if (activity != null) {
                InviteReferralsApi.getInstance(activity).showWelcomeMessage();
            } else {
                Log.i(TAG, "Activity Context found to be Null !!");
            }

        } catch (Exception e) {
            Log.e(TAG, "WELCOME_MSG ERROR :" + e);
        }
    }

    private void get_referrer_code(final Result result) {
        Log.i(TAG, "GET-REFERRER-CODE !!");
        try {
            InviteReferralsApi.getInstance(flutterContext).getReferrerCode(new IRTrackReferrerCode() {
                @Override
                public void getReferrerCode(String response) {
                    if (response != null) {
                        result.success(response);
                        slg("ReferrerCode Response", response);
                    } else {
                        result.success("null");
                        slg("ReferrerCode Response", "null");
                    }
                }
            });
        } catch (Exception e) {
            Log.e(TAG, "GET-REFERRER-CODE ERROR :" + e);
        }
    }

    private void set_language(MethodCall fetchData) {
        Log.i(TAG, "SET-LANGUAGE !!");
        String locale = null;
        try {
            locale = fetchData.argument("locale");
        } catch (Exception e) {
            Log.e(TAG, "ERROR :" + e);
        }

        try {
            InviteReferralsApi.getInstance(flutterContext).setLocale(locale);
        } catch (Exception e) {
            Log.e(TAG, "SET-LANGUAGE ERROR :" + e);
        }
    }

    private void getReferringParams(Result result) {
        Log.i(TAG, "GET REFERRING PARAMS !!");
        try {
            JSONObject mParams = InviteReferralsApi.getInstance(flutterContext).getReferringParams();
            if (mParams != null) {
                result.success(mParams.toString());
                slg("Referring Params Response", mParams.toString());
            } else {
                result.success("null");
                slg("Referring Params Response", "null");
            }

        } catch (Exception e) {
            Log.e(TAG, "GET REFERRING PARAMS ERROR :" + e);
        }
    }

    private void trackInvite(MethodCall fetchData) {
        Log.i(TAG, "TRACK INVITE !!");

        String sourceName = "";
        int mCampaignId = 0;

        try {
            sourceName = fetchData.argument("sourceName");
        } catch (Exception e) {
            Log.i(TAG, "POP-UP TYPE ERROR :" + e);
        }

        try {
            mCampaignId = fetchData.argument("campaignId");
        } catch (Exception e) {
            Log.i(TAG, "CAMPAIGN ID ERROR :" + e);
            Log.i(TAG, "DEFAULT CAMPAIGN-ID 0 IS PASSED !!");
        }

        InviteReferralsApi.getInstance(flutterContext).trackInvite(sourceName, mCampaignId);
    }

    private void getLinkInfo() {
        Log.i(TAG, "GET LINK INFO !!");

    }

    private void clearUserDetails() {
        Log.i(TAG, "CLEAR USER DETAILS !!");
        try {
            InviteReferralsApi.getInstance(flutterContext).clearUserDetails();
        } catch (Exception e) {
            Log.e(TAG, "CLEAR USER DETAILS ERROR :" + e);
        }

    }

    private void show_log(MethodCall fetchData) {
        try {
            String choice = fetchData.argument("choice");
            if (choice != null && choice.equalsIgnoreCase("true")) {
                logSignal = true;
            } else {
                logSignal = false;
            }
        } catch (Exception e) {
            Log.i(TAG, "SHOW-LOG ERROR :" + e);
        }
    }

    private void campaignCallbacks() {
        Log.i(TAG, "Campaign Callbacks !!");
    }

    private IRUserData.Builder fetchUserInfo(MethodCall fetchData) {
        IRUserData.Builder userInfo;

        String mName = "";
        String mEmail = "";
        String mMobile = "";
        String mCustomValueOne = "";
        String mCustomValueTwo = "";

        try {
            mName = fetchData.argument("name");
            slg("Name", mName);
        } catch (Exception e) {
            Log.i(TAG, "NAME ERROR :" + e);
        }

        try {
            mEmail = fetchData.argument("email");
            slg("Email", mEmail);
        } catch (Exception e) {
            Log.i(TAG, "EMAIL ERROR :" + e);
        }

        try {
            mMobile = fetchData.argument("mobile");
            slg("Mobile", mMobile);
        } catch (Exception e) {
            Log.i(TAG, "MOBILE ERROR :" + e);
        }

        try {
            mCustomValueOne = fetchData.argument("customValueOne");
            slg("Custom Value One", mCustomValueOne);
        } catch (Exception e) {
            Log.i(TAG, "CUSTOM-VALUE-ONE ERROR :" + e);
        }

        try {
            mCustomValueTwo = fetchData.argument("customValueTwo");
            slg("Custom Value Two", mCustomValueTwo);
        } catch (Exception e) {
            Log.i(TAG, "CUSTOM-VALUE-TWO ERROR :" + e);
        }

        userInfo = new IRUserData.Builder();
        userInfo.setUserName(mName);
        userInfo.setUserEmail(mEmail);
        userInfo.setUserMobile(mMobile);
        userInfo.setCustomValueOne(mCustomValueOne);
        userInfo.setCustomValueTwo(mCustomValueTwo);
        userInfo.build();
        return userInfo;
    }

    private IRCampaignData.Builder fetchCampaignInfo(MethodCall fetchData) {
        IRCampaignData.Builder campaignInfo;

        int campaignId = 0;
        int templateID = 0;

        try {
            campaignId = fetchData.argument("campaignID");
            slg("Campaign-Id", campaignId + "");
        } catch (Exception e) {
            Log.i(TAG, "GET CAMPAIGN ID ERROR : " + e);
            Log.i(TAG, "DEFAULT CAMPAIGN ID 0 IS PASSED !!");
        }

        try {
            templateID = fetchData.argument("templateID");
            slg("Template-Id", templateID + "");
        } catch (Exception e) {
            Log.i(TAG, "GET TEMPLATE ID ERROR : " + e);
            Log.i(TAG, "DEFAULT TEMPLATE ID 0 IS PASSED !!");
        }

        campaignInfo = new IRCampaignData.Builder();
        campaignInfo.setCampaignID(campaignId);
        campaignInfo.setTemplateID(templateID);
        campaignInfo.build();
        return campaignInfo;
    }

    private IREventData.Builder fetchEventInfo(MethodCall fetchData) {
        IREventData.Builder eventInfo;

        String eventName = "";
        String orderID = "";
        int purchaseValue = 0;
        String referrerCode = "";
        String uniqueCode = "";
        String orderCustomValue = "";

        try {
            eventName = fetchData.argument("eventName");
            slg("Event Name", eventName);
        } catch (Exception e) {
            Log.e(TAG, "EVENT-NAME ERROR :" + e);
        }

        try {
            orderID = fetchData.argument("orderID");
            slg("Order-Id", orderID);
        } catch (Exception e) {
            Log.e(TAG, "ORDER-ID ERROR :" + e);
        }

        try {
            String temp = fetchData.argument("purchaseValue");
            if (temp != null) {
                purchaseValue = Math.round(Float.parseFloat(temp));
            }
            slg("Purchase Value", purchaseValue + "");
        } catch (Exception e) {
            Log.e(TAG, "PURCHASE-VALUE ERROR :" + e);
        }

        try {
            referrerCode = fetchData.argument("referrerCode");
            slg("Referrer Code", referrerCode);
        } catch (Exception e) {
            Log.e(TAG, "REFERRER-CODE ERROR :" + e);
        }

        try {
            uniqueCode = fetchData.argument("uniqueCode");
            slg("Unique Code", uniqueCode);
        } catch (Exception e) {
            Log.e(TAG, "UNIQUE-CODE ERROR :" + e);
        }

         try {
            orderCustomValue = fetchData.argument("orderCustomValue");
            slg("Order Custom Value ", orderCustomValue);
        } catch (Exception e) {
            Log.e(TAG, "Order Custom Value ERROR :" + e);
        }

        eventInfo = new IREventData.Builder();
        eventInfo.setEventName(eventName);
        eventInfo.setOrderID(orderID);
        eventInfo.setPurchaseValue(purchaseValue);
        eventInfo.setReferCode(referrerCode);
        eventInfo.setUniqueCode(uniqueCode);
        eventInfo.setOrderCustomValue(orderCustomValue);
        eventInfo.build();
        return eventInfo;
    }

    private void slg(String hint, String msg) {
        try {
            if (logSignal) {
                Log.i(TAG, hint + " : " + msg);
            }
        } catch (Exception e) {
            Log.i(TAG, "LOG ERROR :" + e);
        }
    }

    private void fetchLogSignal() {
        try {
            ApplicationInfo app = flutterContext.getPackageManager().getApplicationInfo(flutterContext.getPackageName(),
                    PackageManager.GET_META_DATA);
            Bundle bundle = app.metaData;
            boolean signal = bundle.getBoolean("ir_show_logs", false);
            if (signal) {
                logSignal = true;
            } else {
                logSignal = false;
            }
        } catch (Exception e) {
            Log.i(TAG, "FETCH LOG SIGNAL ERROR : " + e);
        }
    }

    private void launch_campaign(IRCampaignData.Builder campaignInfo, IRUserData.Builder userInfo) {
        InviteReferralsApi.getInstance(flutterContext).campaign(userInfo, campaignInfo, new OnUIBuild() {
            @Override
            public void onCampaignScreen(JSONObject response) {
                try {
                    if (response != null) {
                        response.put("callbackType", "campaignScreen");
                        channel.invokeMethod("CampaignCallbacks", response.toString());
                        slg("Campaign Screen", response.toString());
                    } else {
                        JSONObject temp = new JSONObject();
                        temp.put("callbackType", "campaignScreen");
                        temp.put("status", "null");
                        channel.invokeMethod("CampaignCallbacks", temp.toString());
                        slg("Campaign Screen", temp.toString());
                    }
                } catch (Exception e) {
                    Log.e(TAG, "CAMPAIGN SCREEN ERROR :" + e);
                }
            }

            @Override
            public void onShareIconClick(JSONObject response) {
                try {
                    if (response != null) {
                        response.put("callbackType", "shareIconClick");
                        channel.invokeMethod("CampaignCallbacks", response.toString());
                        slg("Share Icon Click", response.toString());
                    } else {
                        JSONObject temp = new JSONObject();
                        temp.put("callbackType", "shareIconClick");
                        temp.put("status", "null");
                        channel.invokeMethod("CampaignCallbacks", temp.toString());
                        slg("Share Icon Click", temp.toString());
                    }
                } catch (Exception e) {
                    Log.e(TAG, "SHARING ICON CLICK ERROR :" + e);
                }
            }

            @Override
            public void onCloseButtonClick() {
                try {
                    JSONObject temp = new JSONObject();
                    temp.put("callbackType", "closeButtonClick");
                    temp.put("status", "success");
                    slg("Close Button Click", temp.toString());
                    channel.invokeMethod("CampaignCallbacks", temp.toString());
                } catch (Exception e) {
                    Log.e(TAG, "CLOSE BUTTON CLICK ERROR :" + e);
                }
            }

            @Override
            public void onCaptureLeadFormSubmission(JSONObject response) {
                try {
                    if (response != null) {
                        response.put("callbackType", "leadFormSubmit");
                        slg("Capture Lead Form Submission", response.toString());
                        channel.invokeMethod("CampaignCallbacks", response.toString());
                    } else {
                        JSONObject temp = new JSONObject();
                        temp.put("callbackType", "leadFormSubmit");
                        temp.put("status", "null");
                        slg("Capture Lead Form Submission", temp.toString());
                        channel.invokeMethod("CampaignCallbacks", temp.toString());
                    }
                } catch (Exception e) {
                    Log.e(TAG, "Capture Lead Form Submission Error : " + e);
                }
            }
        });
    }

}
