#import "InviteReferralsPlugin.h"

BOOL wasDelegateSet = NO;

FlutterResult trackingResult;

@implementation InviteReferralsPlugin


+ (instancetype)sharedInstance {
    static InviteReferralsPlugin *sharedInstance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedInstance = [InviteReferralsPlugin new];
    });
    return sharedInstance;
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    NSLog(@"%@ REGISTER WITH REGISTRAR !!", TAG);
    NSLog(@"FLUTTER INVITEREFERRALS PLUGIN_VERSION : %@ !!", PLUGIN_VERSION);
    InviteReferralsPlugin.sharedInstance.channel  = [FlutterMethodChannel
                                                     methodChannelWithName:@"flutter_invitereferrals"
                                                     binaryMessenger:[registrar messenger]];
    //InviteReferralsPlugin* instance = [[InviteReferralsPlugin alloc] init];
    [registrar addMethodCallDelegate:InviteReferralsPlugin.sharedInstance channel:InviteReferralsPlugin.sharedInstance.channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([INLINE_BUTTON isEqualToString:call.method]) {
        [self inline_btn:call withResult:result];
    } else if([LOGIN_SCREEN isEqualToString:call.method]){
        [self login_screen:call withResult:result];
    } else if([TRACKING isEqualToString:call.method]){
        [self tracking:call withResult:result];
    } else if([CAMPAIGN_POPUP isEqualToString:call.method]){
        [self campaignPopup:call withResult: result];
    } else if([WELCOME_MSG isEqualToString:call.method]){
        [self welcomeMessage:call withResult:result];
    } else if([REFERRER_CODE isEqualToString:call.method]){
        [self getReferrerCode:call withResult:result];
    } else if([SET_LANGUAGE isEqualToString:call.method]){
        [self setLocale:call withResult:result];
    } else if([GET_SHARING_DETAILS isEqualToString:call.method]){
        [self getSharingDetails:call withResult:result];
    } else if([HANDEL_DONE_BUTTON isEqualToString:call.method]){
        [self campaignCallbacksListener:call withResult:result];
    } else if([GET_REFERRING_PARAMS isEqualToString:call.method]){
        [self getReferringParams:call withResult:result];
    } else if([GET_LINK_INFO isEqualToString:call.method]){
        [self getLinkInfo:call withResult:result];
    } else if([SHOW_POP_UP isEqualToString:call.method]){
        [self showPopUp:call withResult:result];
    } else if([LOGOUT_USER isEqualToString:call.method]){
        [self clearUserDetails:call withResult:result];
    } else if([TRACK_INVITE isEqualToString:call.method]){
        [self trackInvite:call withResult:result];
    } else {
        result(FlutterMethodNotImplemented);
    }
}

/* campaign page */
- (void) inline_btn:(FlutterMethodCall *)call withResult:(FlutterResult)result {
    NSLog(@"FLUTTER INVITEREFERRALS : CAMPAIGN SCREEN !!!");
    
    IRCampaignInfo *campaignData = [self fetchCampaignInfo:call];
    IRUserInfo *userData = [self fetchUserInfo:call];
    
    @try{
        [[InviteReferrals sharedInstance] campaign: campaignData userInfo: userData campaignCompletion:^(NSDictionary *campaignCallback) {
            
            //NSLog(@"Campaign Screen : %@", campaignCallback);
            
            NSMutableDictionary *infoD = [campaignCallback mutableCopy];
            [infoD setObject: @"campaignScreen" forKey: @"callbackType"];
            NSString * finalData = [self JsonToString:infoD];
            [self.channel invokeMethod:@"CampaignCallbacks" arguments:finalData];
        }];
    }
    @catch(NSException *exception){
        NSLog(@"USER DETAILS ERROR : %@", exception.reason);
    }
}

/* campaign page */
- (void) login_screen:(FlutterMethodCall *)call withResult:(FlutterResult)result {
    NSLog(@"FLUTTER INVITEREFERRALS : LOGIN SCREEN !!!");
    
    IRCampaignInfo *campaignData = [self fetchCampaignInfo:call];
    
    @try{
        [[InviteReferrals sharedInstance] campaign: campaignData userInfo: nil campaignCompletion:^(NSDictionary *campaignCallback) {
            //NSLog(@"Login Screen : %@", campaignCallback);
            
            NSMutableDictionary *infoD = [campaignCallback mutableCopy];
            [infoD setObject: @"campaignScreen" forKey: @"callbackType"];
            NSString * finalData = [self JsonToString:infoD];
            [self.channel invokeMethod:@"CampaignCallbacks" arguments:finalData];
        }];
    }
    @catch(NSException *exception){
        NSLog(@"USER DETAILS ERROR : %@", exception.reason);
    }
}


/* Track your events */
- (void) tracking:(FlutterMethodCall *)call withResult:(FlutterResult)result {
    NSLog(@"%@ : TRACKING !!", TAG);
    trackingResult = result;
    
    IREventInfo *eventData  = [self fetchEventInfo:call];
    IRUserInfo *userData = [self fetchUserInfo:call];
    
    @try{
        [self setIRDelegate];
        [[InviteReferrals sharedInstance] trackingforEventInfo: eventData userInfo: userData];
        
    }
    @catch(NSException *exception){
        NSLog(@"TRACKING ERROR : %@", exception.reason);
    }
    
}

/* Invite */
- (void) campaignPopup:(FlutterMethodCall *)call withResult:(FlutterResult)result {
    NSLog(@"%@ : CAMPAIGN POPUP !!", TAG);
    
    IRCampaignInfo *campaignData = [self fetchCampaignInfo:call];
    IRUserInfo *userData = [self fetchUserInfo:call];
    
    NSString * mRule;
    
    @try{
        mRule = call.arguments[@"rule"];
        if (mRule == (id)[NSNull null] || mRule.length == 0 ){
            mRule = nil;
        }
    }
    @catch(NSException *exception){
        NSLog(@"RULE ERROR : %@", exception.reason);
    }
    
    @try{
        [[InviteReferrals sharedInstance] campaignPopupForRuleName: mRule campaignInfo: campaignData userInfo: userData];
    }
    @catch(NSException *exception){
        NSLog(@"CAMPAIGN POPUP ERROR : %@", exception.reason);
    }
}

- (void) getSharingDetails:(FlutterMethodCall *)call withResult:(FlutterResult)result {
    NSLog(@"%@ : GET SHARING DETAILS !!", TAG);
    
    IRCampaignInfo *campaignData = [self fetchCampaignInfo:call];
    IRUserInfo *userData = [self fetchUserInfo:call];
    
    @try{
        [[InviteReferrals sharedInstance] getSharingDetailsWithCampaignInfo:campaignData userInfo: userData sharingData:^(NSDictionary *irSharingContent) {
            NSLog(@"Sharing Content : %@", irSharingContent);
            if([irSharingContent count] > 0 ){
                NSMutableDictionary * info = [irSharingContent mutableCopy];
                NSString * irResponseStr = [self JsonToString:info];
                result(irResponseStr);
            }else{
                result(@"null");
            }
            
        }];
    }
    @catch(NSException *exception){
        NSLog(@"GET SHARING DETAILS ERROR : %@", exception.reason);
    }
}

/* Show welcome msg*/
- (void) welcomeMessage:(FlutterMethodCall *)call withResult:(FlutterResult)result {
    NSLog(@"%@ : WELCOME MESSAGE !!", TAG);
    @try{
        [[InviteReferrals sharedInstance] showWelcomeMessage];
    }
    @catch(NSException *exception){
        NSLog(@"WELCOME MESSAGE ERROR : %@", exception.reason);
    }
}

/* Get Referrer Code*/
- (void) getReferrerCode:(FlutterMethodCall *)call withResult:(FlutterResult)result {
    NSLog(@"%@ : GET REFERRER CODE !!", TAG);
    @try{
        [[InviteReferrals sharedInstance]  getReferrerCode:^(NSString * referrerCode) {
            if([referrerCode length] > 0 ){
                result(referrerCode);
            }else{
                result(@"null");
            }
        }];
    }
    @catch(NSException *exception){
        NSLog(@"GET REFERRER CODE ERROR : %@", exception.reason);
    }
}

/* Set App Language if Awailable*/
- (void) setLocale:(FlutterMethodCall *)call withResult:(FlutterResult)result {
    NSLog(@"%@ : SET LANGUAGE !!", TAG);
    NSString * languageCode = nil;
    
    @try{
        languageCode = call.arguments[@"locale"];
        if (languageCode == (id)[NSNull null] || languageCode.length == 0 ){
            languageCode = nil;
        }
    }
    @catch(NSException *exception){
        NSLog(@"LANGUAGE CODE ERROR : %@", exception.reason);
    }
    
    @try{
        [[InviteReferrals sharedInstance] setLocalizationLanguage:languageCode];
    }
    @catch(NSException *exception){
        NSLog(@"SET LANGUAGE ERROR : %@", exception.reason);
    }
}

/* Get Referring Params */
- (void) getReferringParams:(FlutterMethodCall *)call withResult:(FlutterResult)result {
    NSLog(@"%@ : GET REFERRING PARAMS !!", TAG);
    @try{
        [[InviteReferrals sharedInstance] getReferringParams:^(NSDictionary * irRefParams) {
            if([irRefParams count] > 0 ){
                NSError * irErr;
                NSData * irResponse = [NSJSONSerialization dataWithJSONObject: irRefParams options: 0 error: &irErr];
                NSString * irResponseStr = [[NSString alloc] initWithData: irResponse encoding: NSUTF8StringEncoding];
                result(irResponseStr);
            } else{
                result(@"null");
            }
        }];
    }
    @catch(NSException *exception){
        NSLog(@"GET REFERRING PARAMS ERROR : %@", exception.reason);
    }
}


/* get Link Info */
- (void) clearUserDetails:(FlutterMethodCall *)call withResult:(FlutterResult)result {
    NSLog(@"%@ : CLEAR USER DETAILS !!", TAG);
    @try{
        [[InviteReferrals sharedInstance] logout];
    }
    @catch(NSException *exception){
        NSLog(@"GET LINK INFO ERROR : %@", exception.reason);
    }
}

/* get Link Info */
- (void) trackInvite:(FlutterMethodCall *)call withResult:(FlutterResult)result {
    NSLog(@"%@ : TRACK INVITE !!", TAG);
    
    NSString* sourceName = @"";
    NSInteger campaignID = 0;
    
    @try{
        sourceName = call.arguments[@"sourceName"];
        //NSLog(@"Name : %@", sourceName);
    }
    @catch(NSException *exception){
        NSLog(@"Source Name ERROR : %@", exception.reason);
    }
    
    @try{
        if([NSNull null] != call.arguments[@"campaignId"]) {
             campaignID = [call.arguments[@"campaignId"] intValue];
             //NSLog(@"CAMPAIGN-ID : %ld", (long)campaignID);
            //campaignId = [NSString stringWithFormat:@"%d", tmp];
        } else{
            NSLog(@"%@ : DEFAULT CAMPAIGN ID 0 IS PASSED !!", TAG);
            campaignID = 0;
        }
        
    } @catch(NSException *exception){
        NSLog(@"CAMPAIGN ID ERROR : %@", exception.reason);
        NSLog(@"%@ : DEFAULT CAMPAIGN ID 0 IS PASSED !!", TAG);
    }
    
    @try{
        [[InviteReferrals sharedInstance] trackInviteWithSourceName:sourceName campaignID:campaignID];
    }
    @catch(NSException *exception){
        NSLog(@"GET LINK INFO ERROR : %@", exception.reason);
    }
}



/* Close Button Callback */
- (void) campaignCallbacksListener:(FlutterMethodCall *)call withResult:(FlutterResult)result {
    NSLog(@"%@ : CAMPAIGN CALLBACK LISTENER !!", TAG);
    @try{
        [self setIRDelegate];
    }
    @catch(NSException *exception){
        NSLog(@"CAMPAIGN CALLBACK LISTENER ERROR : %@", exception.reason);
    }
}

/* Close Button callback event */
-(void)InviteReferralsOnDoneButtonActionWithUserInfo:(NSMutableDictionary *)userInfo{
    @try {
        [self setIRDelegate];
        if (userInfo.count > 0 ){
            NSMutableDictionary *infoD = [userInfo mutableCopy];
            [infoD setObject: @"closeButtonClick" forKey: @"callbackType"];
            NSString * finalData = [self JsonToString:infoD];
            [self.channel invokeMethod:@"CampaignCallbacks" arguments:finalData];
        } else {
            NSMutableDictionary *infoD = [[NSMutableDictionary alloc] init];
            [infoD setObject: @"success" forKey: @"status"];
            [infoD setObject: @"closeButtonClick" forKey: @"callbackType"];
            NSString * finalData = [self JsonToString:infoD];
            [self.channel invokeMethod:@"CampaignCallbacks" arguments:finalData];
        }
    }@catch(NSException *exception){
        NSLog(@"%@", exception.reason);
    }
}

-(void)InviteReferralsOnSharingIconClickCallback:(NSDictionary *)sharingIconClickInfo {
    [self setIRDelegate];
    //NSLog(@"%@", sharingIconClickInfo);
    NSMutableDictionary *infoD = [sharingIconClickInfo mutableCopy];
    [infoD setObject: @"shareIconClick" forKey: @"callbackType"];
    NSString * finalData = [self JsonToString:infoD];
    [self.channel invokeMethod:@"CampaignCallbacks" arguments:finalData];
}

-(void)InviteReferralsLeadSentResponseCallback:(NSDictionary *)sentLeadResponse {
    [self setIRDelegate];
    //NSLog(@"%@", sentLeadResponse);
    NSMutableDictionary *infoD = [sentLeadResponse mutableCopy];
    [infoD setObject: @"leadFormSubmit" forKey: @"callbackType"];
    NSString * finalData = [self JsonToString:infoD];
    [self.channel invokeMethod:@"CampaignCallbacks" arguments:finalData];
}


-(NSString *)JsonToString: (NSMutableDictionary *)info {
    NSError *nvError = nil;
    NSData *nvJsonData = [NSJSONSerialization dataWithJSONObject: info options: NSJSONWritingPrettyPrinted error: &nvError];
    NSString *nvJsonString = [[NSString alloc] initWithData: nvJsonData encoding: NSUTF8StringEncoding];
    return nvJsonString;
}


/* tracking callback event */
- (void)InviteReferralsTrackingFinishedforEventInfo:(NSDictionary *)trackingResponse{
    @try {
        NSLog(@"INVITEREFERRS TRACKING FINISHED FOR EVENT NAME  !!");
        NSError * irErr;
        NSData * irResponseJsonData = [NSJSONSerialization dataWithJSONObject: trackingResponse options: 0 error: &irErr];
        NSString * irResponseStr = [[NSString alloc] initWithData: irResponseJsonData encoding: NSUTF8StringEncoding];
        trackingResult(irResponseStr);
    }@catch(NSException *exception){
        NSLog(@"%@", exception.reason);
    }
}


/* set Inviteferrals Delegate */
-(void)setIRDelegate{
    @try {
        //NSLog(@"SET IR DELEGATE!!");
        if(!wasDelegateSet){
            [InviteReferrals sharedInstance].delegate = self;
            wasDelegateSet = YES;
        } else{
            //NSLog(@"IR DELEGATE ALLREADY SET !!");
        }
    }@catch(NSException *exception){
        NSLog(@"%@", exception.reason);
    }
    
}

/* get Link Info */
- (void) getLinkInfo:(FlutterMethodCall *)call withResult:(FlutterResult)result {
    NSLog(@"%@ : GET LINK INFO !!", TAG);
    @try{
        
    }
    @catch(NSException *exception){
        NSLog(@"GET LINK INFO ERROR : %@", exception.reason);
    }
}


/* Not Awailable in ios */
- (void) showPopUp:(FlutterMethodCall *)call withResult:(FlutterResult)result {
    NSLog(@"%@ : SHOW POP UP !!", TAG);
    @try{
        
    }
    @catch(NSException *exception){
        NSLog(@"SHOW POP UP ERROR : %@", exception.reason);
    }
}


- (IRUserInfo *) fetchUserInfo:(FlutterMethodCall *)call {
    NSLog(@"%@ : FETCH USER INFO !!", TAG);
    
    IRUserInfo *userData = [[IRUserInfo alloc] init];
    
    @try{
        //self.mName= call.arguments[@"name"];
        userData.name = call.arguments[@"name"];
        if (self.mName == (id)[NSNull null] || self.mName.length == 0 ){
            self.mName  = nil;
        }
        //NSLog(@"Name : %@", userData.name);
    }
    @catch(NSException *exception){
        NSLog(@"NAME ERROR : %@", exception.reason);
    }
    
    @try{
        //self.mEmail = call.arguments[@"email"];
        userData.email = call.arguments[@"email"];
        if (self.mEmail == (id)[NSNull null] || self.mEmail.length == 0 ){
            self.mEmail  = nil;
        }
       // NSLog(@"EMail : %@", userData.email);
    }
    @catch(NSException *exception){
        NSLog(@"EMAIL ERROR : %@", exception.reason);
    }
    
    @try{
        //self.mMobile = call.arguments[@"mobile"];
        userData.mobileNo = call.arguments[@"mobile"];
        if (self.mMobile == (id)[NSNull null] || self.mMobile.length == 0 ){
            self.mMobile  = nil;
        }
        //NSLog(@"Mobile : %@", userData.email);
    }
    @catch(NSException *exception){
        NSLog(@"MOBILE ERROR : %@", exception.reason);
    }
    
    @try{
        //self.mCustomValueOne = call.arguments[@"customValueOne"];
        userData.customValueOne = call.arguments[@"customValueOne"];
        if (self.mCustomValueOne == (id)[NSNull null] || self.mCustomValueOne.length == 0 ){
            self.mCustomValueOne = nil ;
        }
        //NSLog(@"Custom-Value-One : %@", userData.customValueOne);
    }
    @catch(NSException *exception){
        NSLog(@"CUSTOM-VALUE-ONE ERROR : %@", exception.reason);
    }
    
    @try{
        //self.mCustomValueTwo = call.arguments[@"customValueTwo"];
        userData.customValueTwo = call.arguments[@"customValueTwo"];
        if (self.mCustomValueTwo == (id)[NSNull null] || self.mCustomValueTwo.length == 0 ){
            self.mCustomValueTwo = nil ;
        }
        //NSLog(@"Custom-Value-Two : %@", userData.customValueTwo);
    }
    @catch(NSException *exception){
        NSLog(@"CUSTOM-VALUE-TWO ERROR : %@", exception.reason);
    }
    
    return userData;
}

- (IRCampaignInfo *) fetchCampaignInfo:(FlutterMethodCall *)call {
    NSLog(@"%@ : CAMPAIGN INFO !!", TAG);
    
    IRCampaignInfo * campaignData = [[IRCampaignInfo alloc] init];
    campaignData.campaignID = 0;
    campaignData.templateID = 0;
    
    @try{
        if([NSNull null] != call.arguments[@"campaignId"]) {
            campaignData.campaignID = [call.arguments[@"campaignId"] intValue];
            //NSLog(@"CAMPAIGN-ID : %ld", (long)campaignData.campaignID);
            //campaignId = [NSString stringWithFormat:@"%d", tmp];
        } else{
            NSLog(@"%@ : DEFAULT CAMPAIGN ID 0 IS PASSED !!", TAG);
            campaignData.campaignID = 0;
        }
        
    } @catch(NSException *exception){
        NSLog(@"CAMPAIGN ID ERROR : %@", exception.reason);
        NSLog(@"%@ : DEFAULT CAMPAIGN ID 0 IS PASSED !!", TAG);
    }
    
    @try{
        if([NSNull null] != call.arguments[@"templateID"]) {
            campaignData.templateID = [call.arguments[@"templateID"] intValue];
            //NSLog(@"TEMPLATE-ID: %ld", (long)campaignData.templateID);
        } else{
            NSLog(@"%@ : DEFAULT TEMPLATE-ID 0 IS PASSED !!", TAG);
            campaignData.templateID = 0;
        }
        
    } @catch(NSException *exception){
        NSLog(@"TEMPLATE-ID ERROR : %@", exception.reason);
        NSLog(@"%@ : DEFAULT TEMPLATE-ID 0 IS PASSED !!", TAG);
    }
    
    return  campaignData;
}

- (IREventInfo *) fetchEventInfo:(FlutterMethodCall *)call {
    NSLog(@"%@ : EVENT INFO !!", TAG);
    
    IREventInfo * eventData = [[IREventInfo alloc] init];
    
    @try{
        eventData.eventName = call.arguments[@"eventName"];
        if (eventData.eventName == (id)[NSNull null] || eventData.eventName == 0 ){
            eventData.eventName = nil;
        }
    }
    @catch(NSException *exception){
        NSLog(@"NAME ERROR : %@", exception.reason);
    }
    
    @try{
        eventData.orderID = call.arguments[@"orderID"];
        if (eventData.orderID == (id)[NSNull null] || eventData.orderID.length == 0 ){
            eventData.orderID = nil;
        }
    }
    @catch(NSException *exception){
        NSLog(@"ORDER-ID ERROR : %@", exception.reason);
    }
    
    @try{
        if([NSNull null] != call.arguments[@"purchaseValue"]) {
            eventData.purchaseValue = [call.arguments[@"purchaseValue"] intValue];
        } else{
            NSLog(@"%@ : PURCHASE VALUE  0 IS PASSED !!", TAG);
            eventData.purchaseValue = 0;
        }
        
    } @catch(NSException *exception){
        NSLog(@"PURCHASE VALUE ERROR : %@", exception.reason);
        NSLog(@"%@ : PURCHASE VALUE  0 IS PASSED !!", TAG);
    }
    
    
    @try{
        eventData.referrerCode = call.arguments[@"referCode"];
        if (eventData.referrerCode == (id)[NSNull null] || eventData.referrerCode.length == 0 ){
            eventData.referrerCode = nil;
        }
    }
    @catch(NSException *exception){
        NSLog(@"REFER CODE ERROR : %@", exception.reason);
    }
    
    @try{
        eventData.uniqueCode = call.arguments[@"uniqueCode"];
        if (eventData.uniqueCode == (id)[NSNull null] || eventData.uniqueCode.length == 0 ){
            eventData.uniqueCode = nil;
        }
    }
    @catch(NSException *exception){
        NSLog(@"UNIQUE CODE ERROR : %@", exception.reason);
    }


     @try{
        eventData.orderCustomValue = call.arguments[@"orderCustomValue"];
         //NSLog(@"OrderCustomValue : %@", eventData.orderCustomValue);
        if (eventData.orderCustomValue == (id)[NSNull null] || eventData.orderCustomValue.length == 0 ){
            eventData.orderCustomValue = nil;
        }
    }
    @catch(NSException *exception){
        NSLog(@"OrderCustomValue : %@", exception.reason);
    }
    
    return eventData;;
}
    
    -(void) slg: (NSString *)msg{
        NSLog(msg, TAG);
    }

@end
