import 'dart:async';
import 'IRUserInfo.dart';
import 'IRCampaignInfo.dart';
import 'IREventInfo.dart';

import 'package:flutter/services.dart';

// Handlers for various events
typedef void CampaignCallbacks(String msg);

class Invitereferrals {
  static Invitereferrals shared = new Invitereferrals();

  IRCampaignInfo? campaignData;
  IRUserInfo? userData;
  IREventInfo? eventInfo;

  MethodChannel _channel = const MethodChannel('flutter_invitereferrals');

  // event handlers
  CampaignCallbacks? _campaignCallbacks;

  // constructor method
  Invitereferrals() {
    this._channel.setMethodCallHandler(_handleMethod);
  }

  Future<void> campaign(campaignData, userData) async {
    int campaignID = campaignData.campaignID;
    int templateID = campaignData.templateID;

    if (userData != null) {
      String name = userData.name;
      String email = userData.email;
      String mobile = userData.mobile;
      String customValueOne = userData.customValueOne;
      String customValueTwo = userData.customValueTwo;

      var iRInfo = {
        'campaignID': campaignID,
        'templateID': templateID,
        'name': name,
        'email': email,
        'mobile': mobile,
        'customValueOne': customValueOne,
        'customValueTwo': customValueTwo
      };
      await _channel.invokeMethod('inline_btn', iRInfo);
    } else {
      var iRInfo = {
        'campaignID': campaignID,
        'templateID': templateID,
      };
      await _channel.invokeMethod('login_screen', iRInfo);
    }
  }

  Future<String> tracking(eventInfo, userData) async {
    var iRInfo = {};

    if (userData != null) {
      iRInfo = {
        'eventName': eventInfo.eventName,
        'orderID': eventInfo.orderID,
        'purchaseValue': eventInfo.purchaseValue,
        'referrerCode': eventInfo.referrerCode,
        'uniqueCode': eventInfo.uniqueCode,
        'orderCustomValue': eventInfo.orderCustomValue,
        'name': userData.name,
        'email': userData.email,
        'mobile': userData.mobile,
        'customValueOne': userData.customValueOne,
        'customValueTwo': userData.customValueTwo,
      };
    } else {
      iRInfo = {
        'eventName': eventInfo.eventName,
        'orderID': eventInfo.orderID,
        'purchaseValue': eventInfo.purchaseValue,
        'referrerCode': eventInfo.referrerCode,
        'uniqueCode': eventInfo.uniqueCode,
        'orderCustomValue': eventInfo.orderCustomValue,
      };
    }

    String response = await _channel.invokeMethod('tracking', iRInfo);
    return response;
  }

  Future<void> campaignPopup(String ruleName, userData, campaignData) async {
    var iRInfo = {
      'rule': ruleName,
      'campaignID': campaignData.campaignID,
      'templateID': campaignData.templateID,
      'name': userData.name,
      'email': userData.email,
      'mobile': userData.mobile,
      'customValueOne': userData.customValueOne,
      'customValueTwo': userData.customValueTwo
    };
    await _channel.invokeMethod('campaignPopup', iRInfo);
  }

  Future<String> getSharingDetails(userData, campaignData) async {
    var iRInfo = {
      'name': userData.name,
      'email': userData.email,
      'mobile': userData.mobile,
      'customValueOne': userData.customValueOne,
      'customValueTwo': userData.customValueTwo,
      'campaignID': campaignData.campaignID,
      'templateID': campaignData.templateID
    };

    String response = await _channel.invokeMethod('getSharingDetails', iRInfo);
    return response;
  }

  Future<void> welcomeMessage() async {
    await _channel.invokeMethod('welcomeMessage');
  }

  Future<String> getReferrerCode() async {
    String response = await _channel.invokeMethod('getReferrerCode');
    return response;
  }

  Future<void> setLocale(String locale) async {
    var localeInfo = {'locale': locale};
    await _channel.invokeMethod('setLocale', localeInfo);
  }

  Future<String> getReferringParams() async {
    String response = await _channel.invokeMethod('getReferringParams');
    return response;
  }

  Future<void> trackInvite(String sourceName, int campaignID) async {
    var popupInfo = {'sourceName': sourceName, 'campaignId': campaignID};
    await _channel.invokeMethod('trackInvite', popupInfo);
  }

  Future<void> getLinkInfo() async {
    await _channel.invokeMethod('getLinkInfo');
  }

  Future<void> logoutUser() async {
    await _channel.invokeMethod('clearUserDetails');
  }

  Future<void> showLogs(String choice) async {
    await _channel.invokeMethod('showLog', {'choice': choice});
  }

  Future<void> campaignCallbacksListener(CampaignCallbacks handler) async {
    _campaignCallbacks = handler;
    await _channel.invokeMethod('campaignCallbacksListener');
  }

  // Private function that gets called by ObjC/Java
  Future<Null> _handleMethod(MethodCall call) async {
    if (call.method == "CampaignCallbacks" && this._campaignCallbacks != null) {
      this._campaignCallbacks!(call.arguments.toString());
    }
    return null;
  }
}
